@@ -192,7 +192,12 @@ class BaremetalStandaloneManager(bm.BaremetalScenarioTest,
         """
         vifs = cls.get_node_vifs(node_id)
         for vif in vifs:
-            cls.baremetal_client.vif_detach(node_id, vif)
+            try:
+                cls.baremetal_client.vif_detach(node_id, vif)
+            except lib_exc.BadRequest:
+                # When the vif was already removed, such as the
+                # node was already unprovisioned.
+                pass
             if force_delete:
                 try:
                     cls.ports_client.delete_port(vif)
@@ -545,6 +550,21 @@ class BaremetalStandaloneScenarioTest(BaremetalStandaloneManager):
     # The node driver to use in the test
     driver = None
 
+    # If we do not require an explicit driver to be set as a base for
+    # the test. This is useful for tests where we perform a vertical
+    # slice of drivers to test as opposed to focusing all testing
+    # around the driver itself. This allows for us to verify the
+    # driver interfaces align with the established contracts,
+    # and allows the same test to use ipmi, redfish, or some other
+    # known driver to work.
+    use_available_driver = None
+
+    # If we don't require an explicit driver, then what drivers *can* we
+    # operate with. In essence, this exists to prevent the test from failing
+    # on 3rd party drivers, and vendor specific driers which do not support
+    # the sort of itnerfaces we may be trying to test by default.
+    valid_driver_list = []
+
     # The bios interface to use by the HW type. The bios interface of the
     # node used in the test will be set to this value. If set to None, the
     # node will retain its existing bios_interface value (which may have been
@@ -601,7 +621,8 @@ class BaremetalStandaloneScenarioTest(BaremetalStandaloneManager):
     @classmethod
     def skip_checks(cls):
         super(BaremetalStandaloneScenarioTest, cls).skip_checks()
-        if (cls.driver not in CONF.baremetal.enabled_drivers
+        if (not cls.use_available_driver
+            and cls.driver not in CONF.baremetal.enabled_drivers
                 + CONF.baremetal.enabled_hardware_types):
             raise cls.skipException(
                 'The driver: %(driver)s used in test is not in the list of '
@@ -716,6 +737,12 @@ class BaremetalStandaloneScenarioTest(BaremetalStandaloneManager):
 
         # just get an available node
         cls.node = cls.get_and_reserve_node()
+        if (cls.use_available_driver
+                and not cls.driver
+                and cls.node['driver'] in cls.valid_driver_list):
+            # If we're attempting to re-use the existing driver, then
+            # lets save a value for update_node_driver to work with.
+            cls.driver = cls.node['driver']
         cls.update_node_driver(cls.node['uuid'], cls.driver, **boot_kwargs)
 
     @classmethod
@@ -769,8 +796,12 @@ class BaremetalStandaloneScenarioTest(BaremetalStandaloneManager):
                     "args": {"raid_config": config},
                 }
             ]
-            self.baremetal_client.create_deploy_template(
+            _, template = self.baremetal_client.create_deploy_template(
                 'CUSTOM_RAID', steps=steps)
+            # Set a cleanup to ensure the deploy template is removed.
+            self.addCleanup(
+                self.baremetal_client.delete_deploy_template,
+                template['uuid'])
             self.baremetal_client.add_node_trait(self.node['uuid'],
                                                  'CUSTOM_RAID')
 
@@ -851,8 +882,7 @@ class BaremetalStandaloneScenarioTest(BaremetalStandaloneManager):
         self.assertTrue(self.ping_ip_address(self.node_ip,
                                              should_succeed=True))
 
-    @classmethod
-    def boot_node_ramdisk(cls, ramdisk_ref, iso=False):
+    def boot_node_ramdisk(self, ramdisk_ref, iso=False):
         """Boot ironic using a ramdisk node.
 
         The following actions are executed:
@@ -868,11 +898,11 @@ class BaremetalStandaloneScenarioTest(BaremetalStandaloneManager):
                     us actually an ISO image.
         """
         if ramdisk_ref is None:
-            ramdisk_ref = cls.image_ref
+            ramdisk_ref = self.image_ref
 
-        network, subnet, router = cls.create_networks()
-        n_port = cls.create_neutron_port(network_id=network['id'])
-        cls.vif_attach(node_id=cls.node['uuid'], vif_id=n_port['id'])
+        network, subnet, router = self.create_networks()
+        n_port = self.create_neutron_port(network_id=network['id'])
+        self.vif_attach(node_id=self.node['uuid'], vif_id=n_port['id'])
         if iso:
             patch_path = '/instance_info/boot_iso'
         else:
@@ -883,22 +913,25 @@ class BaremetalStandaloneScenarioTest(BaremetalStandaloneManager):
         patch = [{'path': patch_path,
                   'op': 'add',
                   'value': ramdisk_ref}]
-        cls.update_node(cls.node['uuid'], patch=patch)
-        cls.set_node_provision_state(cls.node['uuid'], 'active')
+        self.update_node(self.node['uuid'], patch=patch)
+        self.set_node_provision_state(self.node['uuid'], 'active')
+        self.addCleanup(
+            self.set_node_provision_state,
+            self.node['uuid'], 'deleted')
         if CONF.validation.connect_method == 'floating':
-            cls.node_ip = cls.add_floatingip_to_node(cls.node['uuid'])
+            self.node_ip = self.add_floatingip_to_node(self.node['uuid'])
         elif CONF.validation.connect_method == 'fixed':
-            cls.node_ip = cls.get_server_ip(cls.node['uuid'])
+            self.node_ip = self.get_server_ip(self.node['uuid'])
         else:
             m = ('Configuration option "[validation]/connect_method" '
                  'must be set.')
             raise lib_exc.InvalidConfiguration(m)
-        cls.wait_power_state(cls.node['uuid'],
-                             bm.BaremetalPowerStates.POWER_ON)
-        cls.wait_provisioning_state(cls.node['uuid'],
-                                    bm.BaremetalProvisionStates.ACTIVE,
-                                    timeout=CONF.baremetal.active_timeout,
-                                    interval=30)
+        self.wait_power_state(self.node['uuid'],
+                              bm.BaremetalPowerStates.POWER_ON)
+        self.wait_provisioning_state(self.node['uuid'],
+                                     bm.BaremetalProvisionStates.ACTIVE,
+                                     timeout=CONF.baremetal.active_timeout,
+                                     interval=30)
 
     def boot_and_verify_ramdisk_node(self, ramdisk_ref=None, iso=False,
                                      should_succeed=True):
